package algae.gui;

import java.awt.*;
import java.awt.Graphics;
import java.awt.Graphics.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*;

import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;

import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import javax.swing.BorderFactory; 
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import java.io.*;
import java.lang.Math.*;

public final class ImagePanel extends JPanel {

    MainGUI parent=null;

    static float scale = 0.0f;
    static float scaleHalf = 0.0f;

    int width,height;

    int x,y;
    
    Graphics2D big;
    BufferedImage bi;
    Rectangle area;

    boolean FirstTime=true;

    int pheroPos[][] = null;
    int substratePos[][] = null;
    int algaePos[][] = null;
    int selected=-1;

    boolean doubleBuffer = true;
    boolean drawGrid = true;

    public ImagePanel(MainGUI par) {
	x=0; y=0;
	if ((par.config.width>100)||(par.config.height>100))
	   doubleBuffer=false;
	else
	   doubleBuffer=true; 
	parent=par;
    }

    public void setScale(float sc)
    {
    	scale=sc;
    	scaleHalf=sc/3.0f;
    	repaint();
    }

    public void clearPList()
    {
    	pheroPos=null;
    }
    
    public void clearAList()
    {
    	algaePos=null;
    }

    public void clearSList()
    {
    	substratePos=null;
    }

    public void setPheromones(int pos[][]){
	pheroPos=pos;
    }

    public void setSubstrates(int pos[][]){
	substratePos=pos;
    }
	
    public void setAlgae(int sel, int pos[][]){
	selected=sel;
	algaePos=pos;
    }

    public void paintSubstrate(int pos[][]) {
	if (!parent.viewSubstrate())
	    return;
	if (pos==null)
	    return;
        for (int z=0; z<pos.length; z++)
	{
	    x=(int)((float)pos[z][0]*scale);
	    y=(int)((float)pos[z][1]*scale);
	
	    big.setPaint(Color.green);
	    big.fill(new Rectangle2D.Double(x,y,scale,scale));
	}
    } 

    public void paintPheromone(int pos[][]) {
    	if (!parent.viewPheromone())
    	    return;
	if (pos==null)
	    return;
	for (int z=0; z<pos.length; z++)
	{
	    int x=(int)((float)pos[z][0]*scale);
	    int y=(int)((float)pos[z][1]*scale);
	
	    big.setPaint(Color.blue);
	    big.fill(new Rectangle2D.Double(x,y,scale,scale));
	}
    } 
    
    public void paintAlgae(int marked, int pos[][]) { 
    	if (!parent.viewAlgae())
    	    return;
	if (pos==null)
	    return;
       for (int z=0; z<pos.length; z++)
       {
	if (z!=marked)
	    big.setColor(Color.black);
	else
	    big.setColor(Color.red);	
	Polygon p=new Polygon();
	Point2D.Double p2=new Point2D.Double();
	x=(int)(((float)pos[z][0]*scale)+(scale/2));
	y=(int)(((float)pos[z][1]*scale)+(scale/2));
	
	switch (pos[z][2])
	{
	    case 1:
	        big.drawLine((int)(x-scale/2)+1,(int)(y-scale/2)+1,(int)(x+scale/2),y);
		big.drawLine((int)(x+scale/2),y+1,(int)(x-scale/2)+1,(int)(y+scale/2));
		big.drawLine((int)(x-scale/2)+1,(int)(y-scale/2)+1,(int)(x-scale/2)+1,(int)(y+scale/2));
                break;
	    case 2:
		big.drawLine((int)(x-scale/2)+1,(int)(y-scale/2)+1,x,(int)(y+scale/2));
		big.drawLine((int)(x-scale/2)+1,(int)(y-scale/2)+1,(int)(x+scale/2),(int)(y-scale/2)+1);
		big.drawLine((int)(x+scale/2),(int)(y-scale/2)+1,x+1,(int)(y+scale/2));
               	break;
	    case 3:
	   	big.drawLine((int)(x+scale/2),(int)(y-scale/2)+1,(int)(x-scale/2)+1,y);
		big.drawLine((int)(x-scale/2)+1,y+1,(int)(x+scale/2),(int)(y+scale/2));
		big.drawLine((int)(x+scale/2),(int)(y-scale/2)+1,(int)(x+scale/2),(int)(y+scale/2));
                break;
            case 4:
		big.drawLine((int)(x-scale/2)+1,(int)(y+scale/2),x+1,(int)(y-scale/2)+1);
		big.drawLine((int)(x-scale/2)+1,(int)(y+scale/2),(int)(x+scale/2),(int)(y+scale/2));
		big.drawLine((int)(x+scale/2),(int)(y+scale/2),x+1,(int)(y-scale/2)+1);
               	break;
	}
     }
    }

    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        //paintRefresh();
        this.update(g);
    }
  
    public void paintGrid()
    {
	if (!drawGrid)
	    return;
    	if (scale<=4.0f) // draw grid only if gridsize is big enough
    	    return;
    	
    	int zx=0;
    	int zy=0;
    	big.setColor(Color.gray);
    	big.drawRect(0,0,(int)((float)width*scale),(int)((float)height*scale));
    	
    	for (zx=0; zx<width; zx++)
    	     big.drawLine((int)((float)zx*scale),0,
    	                  (int)((float)zx*scale),(int)((float)height*scale));
    	for (zy=0; zy<height; zy++)
    	     big.drawLine(0,(int)((float)zy*scale),
    	                  (int)((float)width*scale),(int)((float)zy*scale));    	                  	
    }
    
    public void turnOnOffDB(boolean dbSwitch)
    {
	doubleBuffer=dbSwitch;
    }

    public void turnOnOffGrid(boolean gSwitch)
    {
	drawGrid=gSwitch;
	this.update(this.getGraphics());
    }

    public void setWidth(int w)
    {
    	width=w;
    }
    
    public void setHeight(int h)
    {
    	height=h;
    }
      
    public void paintAll(){
	paintPheromone(pheroPos);
	paintSubstrate(substratePos);
	paintAlgae(selected,algaePos);
    }

    public void update(Graphics g)
    {
	Graphics2D g2 = (Graphics2D)g;
	int w = (int)((float)width*scale);
	int h = (int)((float)height*scale);
	if (doubleBuffer)
	{
	    Dimension dim = getSize();
	    
	    if (FirstTime)
	    {
		area = new Rectangle(dim);
		bi = (BufferedImage)createImage(1000,1000);
		big = bi.createGraphics();
                 
		FirstTime = false;
	    }
    	    // Clears the rectangle that was previously drawn.
	    big.setColor(Color.white);
	    big.clearRect(0, 0, 1000,1000);
	} 
	else
	{
	    big=(Graphics2D)g;

	    // Clears the rectangle that was previously drawn.
	    big.setColor(Color.white);
	    big.clearRect(0, 0, w,h);
	}
 
 	paintGrid();
 
        // Draws and fills the newly positioned rectangle to the buffer.
        paintAll();

        // Draws the buffered image to the screen.
	if (doubleBuffer)
	    g2.drawImage(bi, 0, 0, this);
    }
    
    public void paintRefresh()
    {
	repaint();
    }
}
