package algae.simulation.agents;

import java.util.*;
import java.util.Random;
import algae.simulation.Settings;
import algae.simulation.worlds.PheromoneWorld;
import algae.simulation.worlds.AgentWorld;
import algae.simulation.worlds.SubstrateWorld;
import algae.simulation.worlds.LightWorld;

/** A class that represents ...
 * 
 * @see OtherClasses
 * @author your_name_here
 */
public class Agent {

    public final static int SENSORCOUNT = 16;

    public GenoType genoType;
    public PhenoType phenoType;

    public AgentSensor sensors[]=new AgentSensor[4];
    
    public PheromoneWorld pheromoneWorld;
    public SubstrateWorld substrateWorld;
    public AgentWorld agentWorld;
    public LightWorld lightWorld;
    
    public int x;
    public int y;
    public int dir;
    
    public boolean dead=false;

    public float energy_level = 0.0f;

    public int ann_output=0;
    public float [] sensor_values = new float[SENSORCOUNT];

    public boolean usedSubstrate = false;
    public boolean usedLight = false;
    public boolean madeMitosis = false;
    public boolean madeMeiosis = false;
    public boolean died = false;
    public boolean born = false;
    
    Settings cfg;
        
    final static float mRate = 0.1f;

    public boolean isAlive()
    {
    	return (!dead);
    }

    // randomly initialized agent
    public Agent(int sx, int sy, Settings c, Random rand,
                 AgentWorld aw, SubstrateWorld sw,
                 PheromoneWorld pw, LightWorld lw)
    {
	cfg=c;
	x=sx;
	y=sy;
	dir=Math.abs(rand.nextInt()%4)+1; 
		
	agentWorld=aw;
	lightWorld=lw;
	substrateWorld=sw;
	pheromoneWorld=pw;
	
	energy_level=cfg.initial_energy;

	genoType = new GenoType(cfg.randSeed);
	phenoType = new PhenoType(genoType);
	
	// bumper sensor
	sensors[0]=new AgentSensor(cfg,cfg.bumper_range,10);
	// pheromone sensor
	sensors[1]=new AgentSensor(cfg,cfg.pheromone_range,cfg.pheromone_fov);
	// substrate sensor
	sensors[2]=new AgentSensor(cfg,cfg.substrate_range,cfg.substrate_fov);
	// light sensor
	sensors[3]=new AgentSensor(cfg,cfg.light_range,cfg.light_fov);
    }

    // meiosis
    public Agent(Agent a, Agent b, float crossOverRate, float mutationRate, Settings c, Random rand)
    {
	cfg=c;
	
	dir=Math.abs(rand.nextInt()%4)+1; 
	
	agentWorld=a.agentWorld;
	lightWorld=a.lightWorld;
	substrateWorld=a.substrateWorld;
	pheromoneWorld=a.pheromoneWorld;
		
	b.energy_level=b.energy_level-cfg.energy_mitose;
	a.energy_level=a.energy_level-cfg.energy_mitose;
	energy_level=cfg.initial_energy;

	genoType = new GenoType(cfg.randSeed, a.genoType, b.genoType, crossOverRate, mRate);
	phenoType = new PhenoType(genoType);
	
	// bumper sensor
	sensors[0]=new AgentSensor(cfg,cfg.bumper_range,10);
	// pheromone sensor
	sensors[1]=new AgentSensor(cfg,cfg.pheromone_range,cfg.pheromone_fov);
	// substrate sensor
	sensors[2]=new AgentSensor(cfg,cfg.substrate_range,cfg.substrate_fov);
	// light sensor
	sensors[3]=new AgentSensor(cfg,cfg.light_range,cfg.light_fov);
    }

    // mitosis
    public Agent(Agent a, float mutationRate, Settings c, Random rand)
    {
	cfg=c;
		
	dir=Math.abs(rand.nextInt()%4)+1; 
		
	agentWorld=a.agentWorld;
	lightWorld=a.lightWorld;
	substrateWorld=a.substrateWorld;
	pheromoneWorld=a.pheromoneWorld;
		
	a.energy_level=a.energy_level-cfg.energy_meiose;
	energy_level=cfg.initial_energy;

	genoType = new GenoType(cfg.randSeed, a.genoType, mRate);
	phenoType = new PhenoType(genoType);
	
	// bumper sensor
	sensors[0]=new AgentSensor(cfg,cfg.bumper_range,10);
	// pheromone sensor
	sensors[1]=new AgentSensor(cfg,cfg.pheromone_range,cfg.pheromone_fov);
	// substrate sensor
	sensors[2]=new AgentSensor(cfg,cfg.substrate_range,cfg.substrate_fov);
	// light sensor
	sensors[3]=new AgentSensor(cfg,cfg.light_range,cfg.light_fov);
    }
    
    public float[] getInfos()
    {
    	float invec[] = new float[18];
	
	float tmp[] = new float[4];

	invec[0]=(float)this.dir;
	invec[1]=this.energy_level;
	
	// get light sensor values
	tmp=sensors[3].getValuesL(x,y,lightWorld);
	for (int z=0; z<4; z++)
	    invec[z+2]=(float)tmp[z];
	
	// get substrate sensor values
	tmp=sensors[2].getValuesS(x,y,substrateWorld);
	for (int z=0; z<4; z++)
	    invec[1*4+z+2]=(float)tmp[z];
	
	// get pheromone sensor values
	tmp=sensors[1].getValuesP(x,y,pheromoneWorld);
	for (int z=0; z<4; z++)
	    invec[2*4+z+2]=(float)tmp[z];
	
	// get bumper sensor values
	tmp=sensors[0].getValuesA(x,y,agentWorld);
	for (int z=0; z<4; z++)
	    invec[3*4+z+2]=(float)tmp[z];
		
	return invec;
    }

    // calculates a step (neural network) and returns the winner neuron
    public int calcStep()
    {
	float invec[] = new float[17];
	invec[0]=energy_level;

	float tmp[] = new float[4];

	// get bumper sensor values
	tmp=sensors[0].getValuesA(x,y,agentWorld);
	for (int z=0; z<4; z++)
	    invec[z+1]=(float)tmp[z];
	
	// get pheromone sensor values
	tmp=sensors[1].getValuesP(x,y,pheromoneWorld);
	for (int z=0; z<4; z++)
	    invec[4+z+1]=(float)tmp[z];
		
	// get substrate sensor values
	tmp=sensors[2].getValuesS(x,y,substrateWorld);
	for (int z=0; z<4; z++)
	    invec[2*4+z+1]=(float)tmp[z];
	
	// get light sensor values
	tmp=sensors[3].getValuesL(x,y,lightWorld);
	for (int z=0; z<4; z++)
	    invec[3*4+z+1]=(float)tmp[z];
	
	// transfer all sensor values to a temporary storage for later use (statistics)
	for (int z=0; z<SENSORCOUNT; z++)
	{
	    sensor_values[z]=invec[1+z];	
	}
	
	ann_output=phenoType.processNet(invec);	
	return ann_output;
    }
    
} /* end class Agent */
