package algae.simulation.worlds;

import java.util.*;
import algae.simulation.Settings;
import algae.simulation.view.ViewLight;


public class LightWorld extends World 
{
    // Attributes
    
    final double baseIntensity;
    public double intensity;
    final int mode;
    AgentWorld aWorld;
    
    class Element extends World.Element
    {
        final double NOTHING = 0.0; 
        int x;
        int y;
        
        // Constructors
        public Element(int x, int y)
            {
                this.x = x;
                this.y = y;
                this.amount = NOTHING;
            }
        
        
        // Operations
        
        public boolean exists()
            {
                return (this.amount > 0);
            }
        
        public boolean add(int s)
        {
        	return true;
        }
        
        public boolean remove(int s)
        {
        	return true;
        }
        
        boolean update()
        {
            boolean noAgentsAbove = true;
            
            for(int i = 0; i < y; i++)
                {
                    if (aWorld.world[i][this.x].exists())
                       noAgentsAbove=false;
                }
            if(noAgentsAbove) 
                {
                    this.amount = intensity * 1.0/(double)(y+1);
                }
            else
            {
            	 this.amount = NOTHING;
            }
            
            return true;
            
        } // update
        
    }// end class Element
    

    public LightWorld(Settings set) 
    {
        super(set.height, set.width);
        System.out.println("LightWorld constructor begin...");

        this.baseIntensity = set.light_intensity;
        this.intensity = 0.0;
        this.mode = set.light_enabled;
        cfg=set;
    
        for(int i = 0; i < set.height;i++) 
            {       
                for(int j = 0; j < set.width;j++) 
                    {
                            this.world[i][j] = new Element(j,i);
                    }
            }
        System.out.println("LightWorld constructor ends here...");
    }//LightWorld


    public void setAgentWorld(AgentWorld aWorld)
    {
        this.aWorld = aWorld;
    }

    public void step(int step)
    {
        switch (this.mode)
            {
            case 0 | 1: 
                intensity = Math.abs(baseIntensity * mode);
            default   :
                intensity = Math.abs(baseIntensity * Math.sin((double)step/100.0));
            }// switch

        for(int i = 0; i < cfg.height;i++) 
            {       
                for(int j = 0; j < cfg.width;j++) 
                    {
                         this.world[i][j].update();
                    }
            }
    }//step
    
    public void update(int fromX, int fromY, int toX, int toY)
    {
        for(int i = fromY; i >= 0; i--)
            {
                world[i][fromX].update();
            }
        
        for(int i = toY; i >= 0; i--)
            {
                world[i][toX].update();
            }           
    } // update

    public double consume(int x, int y)
    {
    	used++;
        return world[y][x].value();
    }// consume
    
    public ViewLight[] getList()
    {
    	ViewLight ret[];
        int retC=0;
        
        for(int i = 0; i < cfg.height;i++) 
            {
                for(int j = 0; j < cfg.width;j++) 
                    {
                        if (this.world[i][j].amount>0.0)
                           retC++;
                    }
            }
           
        int z=0;
        
        if (retC==0)
           return null;
                    
        ret=new ViewLight[retC];
        
        for(int i = 0; i < cfg.height;i++) 
            {
                for(int j = 0; j < cfg.width;j++) 
                    {
                        if(this.world[i][j].amount>0.0)
                        {
                            ret[z]=new ViewLight(j,i,(float)this.world[i][j].amount);
                            z++;
                        }
                    }
            }
            
        return ret;
    }//getList
    
} /* end class LightWorld */